<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities;

class RelProdutosMaisVendidos {

    const ID_RELATORIO_VALOR = 'V',
          ID_RELATORIO_QTDE  = 'Q',
          ID_RELATORIO_CONS  = 'C';
          
    const dataSourceNameQTDE  = 'vnd02400_relProdutosMaisVendidosQTDE',
          dataSourceNameVALOR = 'vnd02400_relProdutosMaisVendidosVALOR',
          dataSourceNameCONS = 'vnd02400_relProdutosMaisVendidosCONSO',
          dataSourceNameDefault = 'vnd02400_relProdutosMaisVendidos';
    protected $utilService,
              $relProdutosMaisVendidos;
    
    public function __construct(Utilities\Util $utilService, $relProdutosMaisVendidos) {
        $this->utilService = $utilService;
        $this->relProdutosMaisVendidos = $relProdutosMaisVendidos;
    }
    
    private function setTCDandCDcondition(&$conditions, $key, $keyName) {
        if( !array_key_exists($key, $conditions) ) {
            $conditions['T'.$keyName] = 'T';    
            $conditions[$keyName] = null;    
        } else {
            if(!empty($conditions[$key])) {
                $conditions[$keyName] = '_'.implode('_', $conditions[$keyName] ).'_';
            }
            $conditions['T'.$keyName] = NULL;
            if($key !== $keyName) {
                unset($conditions[$keyName]);
            }
        }
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response ) {
        $conditions = $this->utilService->getParamsAsArray($request);
            
        if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_VALOR) {
            $this->dataSourceName = self::dataSourceNameVALOR;
        }  else if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_QTDE) {
            $this->dataSourceName = self::dataSourceNameQTDE;
        } else if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_CONS) {
            $this->dataSourceName = self::dataSourceNameCONS;
        } else {
            throw new \Exception ('Tipo de relatório não encontrado!');
        }
        
        if( !array_key_exists('CDFILILOJA', $conditions) ) {
            $conditions['CDFILILOJA'] = null;
        }
        
        if( !array_key_exists('CDFILILOJACAIXA', $conditions) ) {
            $conditions['CDFILILOJACAIXA'] = null;
        }
        $this->setTCDandCDcondition($conditions, 'CDFILIAL', 'CDFILIAL');
        $this->setTCDandCDcondition($conditions, 'CDLOJA', 'CDLOJA');
        $this->setTCDandCDcondition($conditions, 'CDCAIXA', 'CDCAIXA');
        // If you use a different name than DTENTRVENDA, 
        // just copy this line and set yours condition name.
        // Or you could just use an Outdata for DTENTRVENDA.
        if( array_key_exists('DTENTRVENDA', $conditions) ) {
            $conditions['DTINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DTFIN'] = $conditions['DTENTRVENDA'][1];
            unset($conditions['DTENTRVENDA']);
        }
        
        if (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I'){
           if (empty($conditions['CDARVPRODF'])){
                $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
           }
           
           $conditions['CDARVPRODI'] = str_replace("%", "",$conditions['CDARVPRODI']);
           $conditions['CDARVPRODF'] = str_replace("%", "",$conditions['CDARVPRODF']);
           $conditions["CDARVPRODF"]=str_pad($conditions["CDARVPRODF"], 15, 'z');
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['PRODNIV'] = null;
           
        }elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'D') {
           $conditions['PRODNIV'] = str_replace('%', '', str_replace('.', '', $conditions['PRODNIV'])).'%';
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
        }
        elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L'){
           $conditions['CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
        else{
           $conditions['TCDPRODUTO'] = 'T';
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
         
        if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_VALOR) {
            $produtosResult = $this->relProdutosMaisVendidos->getProdutosMaisVendidosPorValor($conditions);
        }  else if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_QTDE) {
            $produtosResult = $this->relProdutosMaisVendidos->getProdutosMaisVendidosPorQtde($conditions);
        } else if($conditions['IDTIPORELATORIO'] === self::ID_RELATORIO_CONS) {
            $produtosResult = $this->relProdutosMaisVendidos->getProdutosMaisVendidosConsolid($conditions);
        }
        
        $response->addDataSet( new DataSet(self::dataSourceNameDefault, $produtosResult));
    }

}
