<?php
namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Util\Functions;

class RelatorioService {
    
    // Receives a Zeedhi's FilterCriteria's Request Object
    // And return an array where keys are columnNames. 
    public function getParamsAsArray($request) {
        $arr = array();
        foreach ($request->getFilterCriteria()->getConditions() as $condition) {
            $arr[ $condition['columnName'] ] = $condition['value'];  
        }
        return $arr;
    }
    
    /* Receives a condition array, a key that exists in that array, and the name you want that key to become.
       Like setTCDandCDcondition($conditions, 'CDCAIXA', 'CDFILICAIXA')
       Input : array('CDCAIXA' => array('0001001', '0002001'))
       Output: array('CDFILICAIXA' => '0001001_0002001_', 'TCDFILICAIXA' => 'F') */
    public function setTCDandCDcondition(&$conditions, $key, $keyName) {
        if(!Functions::arrayKeyExists($key, $conditions) || empty($conditions[$key])) {
            $conditions['T'.$keyName] = 'T';    
            $conditions[$keyName] = null;    
        } else {
            $conditions[$keyName] = '_'.implode('_', $conditions[$keyName] ).'_';
            $conditions['T'.$keyName] = 'F';
            if($key !== $keyName) {
                unset($conditions[$keyName]);
            }
        }
    }
    
    /* Receives a condition array, a key that exists in that array, and the name you want that key to become.
       Like setTCDandCDcondition($conditions, 'CDCAIXA', 'CDFILICAIXA')
       Input : array('CDCAIXA' => array('0001001', '0002001'))
       Output: array('CDFILICAIXA' => '0001001_0002001_', 'TCDFILICAIXA' => 'F') */
    public function setTCDandCDChangingKey(&$conditions, $key, $keyName) {
        if(!Functions::arrayKeyExists($key, $conditions) || empty($conditions[$key])) {
            $conditions['T'.$keyName] = 'T';    
            $conditions[$keyName] = null;    
        } else {
            $conditions[$keyName] = '_'.implode('_', $conditions[$key] ).'_';
            $conditions['T'.$keyName] = 'F';
            if($key !== $keyName) {
                unset($conditions[$key]);
            }
        }
    }
    
    // Receives a array and a key, that exists in the conditions and has a array as value.
    // Returns the TCD and CD conditions without a array, just like above.N 
    public function addParams($conditions, $param) {
        if(!Functions::arrayKeyExists($param, $conditions)) {
            $conditions[$param] = '';
            $conditions['T'.$param] = 'T';
        } else {
            $conditions[$param] = implode('_', $conditions[$param]);
            $conditions['T'.$param] = '';
        }
        return $conditions;
    }
    
    // Returns the names of the conditions that use the operator LIKE
    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' || $condition['operator'] === 'MAPPED_LIKE_ALL' )
                array_push($result, $condition);
        }
        return $result;
    }
    
    // Create a new Zeedhi's FilterCriteria's Request object with the conditions array.
    // Obs: all conditions will have the operator equal ('=') in it.
    public function createNewZeedhiFilterRequestObject($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"   => "=",
                "value"      => $value
            );
        }
        
        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }
    
    // Formats a Interval's field filters params as start date and end date into a conditions array.
    public function setIntervalAsDTINIandDTFIM(&$conditions, $keyDate, $keyStartDate, $keyEndDate) {
        if(Functions::arrayKeyExists($keyDate, $conditions)) {
            $conditions[$keyStartDate] = $conditions[$keyDate][0];
            $conditions[$keyEndDate  ] = $conditions[$keyDate][1];
            unset($conditions[$keyDate]);
        }
    }
    
    // Formats a Interval's field filters params as start date and end date into a conditions array.
    public function setCustomDate(&$conditions, $keyDate, $keyStartDate, $keyEndDate) {
        if(Functions::arrayKeyExists($keyDate, $conditions)) {
            $conditions[$keyStartDate] = $conditions[$keyDate]['start'];
            $conditions[$keyEndDate  ] = $conditions[$keyDate]['end'];
            unset($conditions[$keyDate]);
        }
    }
    
}